module cpu (
   // Outputs
   cpu_irom_addr_i, cpu_ram_addr_e, cpu_ram_data_e, cpu_ram_we_e, 
   // Inputs
   clk, reset, ext_cond, irom_cpu_data_r, ram_cpu_data_w
   ) ;
   input clk;
   input reset;
   input ext_cond;
   input [15:0] irom_cpu_data_r;
   input [11:0] ram_cpu_data_w;

   output [11:0] cpu_irom_addr_i;
   output [11:0] cpu_ram_addr_e;
   output [11:0] cpu_ram_data_e;
   output        cpu_ram_we_e;

   // list internal nets here
   wire [15:0]   inst_r;
   wire [15:0]   inst_e;

/*
 * *********************************************
 *			I-stage
 * *********************************************
 */
   pc pc(.cpu_irom_addr(cpu_irom_addr_i), 
         .inst_r(inst_r), 
         .clk(clk), 
         .pc_next(),
         .pc_short(), 
         .pc_long());

/* *************************************************
 *			I to R flops
 * *************************************************/

/*
 * *********************************************
 *			R-stage
 * *********************************************
 */
   regfile8 regfile(// Outputs
                    .ra_data_r          (),
                    .rb_data_r          (),
                    // Inputs
                    .clk                (clk),
                    .ra_addr_r          (inst_r[5:3]),
                    .rb_addr_r          (inst_r[2:0]),
                    .wc_addr_w          (),
                    .wc_enable_w        (),
                    .wc_data_w          ());

   control control(// Outputs
                   .pc_next             (),
                   .pc_long             (),
                   .pc_short            (),
                   .writesback_r        (),
                   .load_r              (),
                   .store_r             (),
                   // Inputs
                   .clk                 (clk),
                   .reset               (reset),
                   .opcode              (inst_r[15:12]),
                   .cond_sel            (inst_r[11:8]),
                   .alu_mem_op          (inst_r[10:6]),
                   .zero_e              (),
                   .carry_e             (),
                   .neg_e               (),
                   .ext_cond            ());
 

/* *************************************************
 *			R to E flops
 * *************************************************/
   dffr #(16) inst_r2e(.clk(clk), .r(reset),
                       .d(inst_r[15:0]),
                       .q(inst_e[15:0]));

/*
 * *********************************************
 *			E-stage
 * *********************************************
 */

   alu alu(// Outputs
           .zero_e                      (),
           .carry_e                     (),
           .neg_e                       (),
           .alu_out_e                   (),
           // Inputs
           .ra_data_e                   (),
           .rb_data_e                   (),
           .op_e                        ());

/* *************************************************
 *			E to W flops
 * *************************************************/
           
   
  
   
endmodule // cpu
