// module: lab3_top
// =================
// This module hooks up the ram, rom and cpu.
// It also brings in the switch (external) data.
// You should not have to touch this module.

module lab3_top (clk, reset, ext_cond, sw_data) ;

input clk ;
input reset ;
input ext_cond ;
input [11:0] sw_data ;

wire [11:0] cpu_irom_addr_i;
wire [11:0] cpu_ram_addr_e;
wire [11:0] cpu_ram_data_e;
wire        cpu_ram_we_e;
wire [15:0] irom_cpu_data_r;
wire [11:0] ram_cpu_data_w;

wire ram_we_e;
wire [11:0] ram_data_w;
wire use_switch_w;

cpu cpu(.cpu_irom_addr_i(cpu_irom_addr_i[11:0]),
	.cpu_ram_addr_e(cpu_ram_addr_e[11:0]),
	.cpu_ram_data_e(cpu_ram_data_e[11:0]),
	.cpu_ram_we_e(cpu_ram_we_e),
	//inputs
	.clk(clk),
	.reset(reset),
	.ext_cond(ext_cond),
	.irom_cpu_data_r(irom_cpu_data_r[15:0]),
	.ram_cpu_data_w(ram_cpu_data_w[11:0]));
	
irom irom(.addr(cpu_irom_addr_i[6:0]),
	  .clk(clk),
	  .dout(irom_cpu_data_r[15:0]));
	  
data_ram ram(.addr(cpu_ram_data_e[6:0]),
	     .clk(clk),
	     .din(cpu_ram_data_e[11:0]),
	     .dout(ram_data_w[11:0]),
	     .we(ram_we_e));
	     
assign ram_we_e = cpu_ram_we_e & ~cpu_ram_data_e[11];
dff use_switch_dff(.d(cpu_ram_data_e[11]),
		   .clk(clk),
		   .q(use_switch_w));
assign ram_cpu_data_w[11:0] = (use_switch_w)? sw_data[11:0]: ram_data_w[11:0];

endmodule 
