module regfile8 (ra_data_r, rb_data_r, clk, reset, ra_addr_r, rb_addr_r, wc_addr_w, wc_enable_w, wc_data_w) ;

output [11:0] ra_data_r ;
output [11:0] rb_data_r ;
input clk ;
input reset;
input [2:0] ra_addr_r ;
input [2:0] rb_addr_r ;
input [2:0] wc_addr_w ;
input wc_enable_w ;
input [11:0] wc_data_w ;

reg [11:0] ra_data_r,
	   rb_data_r;

wire we_r0,
     we_r1,
     we_r2,
     we_r3,
     we_r4,
     we_r5,
     we_r6,
     we_r7;
     
wire [11:0] r0_data,
	    r1_data,
	    r2_data,
	    r3_data,
	    r4_data,
	    r5_data,
	    r6_data,
	    r7_data;

// generation of write enables	    
assign we_r0 = wc_enable_w & (wc_addr_w[2:0] == 0);
assign we_r1 = wc_enable_w & (wc_addr_w[2:0] == 1);
assign we_r2 = wc_enable_w & (wc_addr_w[2:0] == 2);
assign we_r3 = wc_enable_w & (wc_addr_w[2:0] == 3);
assign we_r4 = wc_enable_w & (wc_addr_w[2:0] == 4);
assign we_r5 = wc_enable_w & (wc_addr_w[2:0] == 5);
assign we_r6 = wc_enable_w & (wc_addr_w[2:0] == 6);
assign we_r7 = wc_enable_w & (wc_addr_w[2:0] == 7);

// storage of registers
dffre #(12) r0(.d(wc_data_w[11:0]), .clk(clk), .r(reset),
	       .en(we_r0), .q(r0_data[11:0]));
dffre #(12) r1(.d(wc_data_w[11:0]), .clk(clk), .r(reset),
	       .en(we_r1), .q(r1_data[11:0]));
dffre #(12) r2(.d(wc_data_w[11:0]), .clk(clk), .r(reset),
	       .en(we_r2), .q(r2_data[11:0]));
dffre #(12) r3(.d(wc_data_w[11:0]), .clk(clk), .r(reset),
	       .en(we_r3), .q(r3_data[11:0]));  
dffre #(12) r4(.d(wc_data_w[11:0]), .clk(clk), .r(reset),
	       .en(we_r4), .q(r4_data[11:0]));
dffre #(12) r5(.d(wc_data_w[11:0]), .clk(clk), .r(reset),
	       .en(we_r5), .q(r5_data[11:0]));
dffre #(12) r6(.d(wc_data_w[11:0]), .clk(clk), .r(reset),
	       .en(we_r6), .q(r6_data[11:0]));
dffre #(12) r7(.d(wc_data_w[11:0]), .clk(clk), .r(reset),
	       .en(we_r7), .q(r7_data[11:0]));
	       
// output assignment
always @ (r0_data or r1_data or r2_data or r3_data or r4_data or r5_data
	 or r6_data or r7_data or ra_addr_r or rb_addr_r) begin
	 
	 case (ra_addr_r[2:0])
	 	3'h0: ra_data_r[11:0] = r0_data[11:0];
	 	3'h1: ra_data_r[11:0] = r1_data[11:0];
	 	3'h2: ra_data_r[11:0] = r2_data[11:0];
	 	3'h3: ra_data_r[11:0] = r3_data[11:0];
	 	3'h4: ra_data_r[11:0] = r4_data[11:0];
	 	3'h5: ra_data_r[11:0] = r5_data[11:0];
	 	3'h6: ra_data_r[11:0] = r6_data[11:0];
	 	3'h7: ra_data_r[11:0] = r7_data[11:0];
	 	default: ra_data_r[11:0] = 12'b0;
	 endcase
	 case (rb_addr_r[2:0])
	 	3'h0: rb_data_r[11:0] = r0_data[11:0];
	 	3'h1: rb_data_r[11:0] = r1_data[11:0];
	 	3'h2: rb_data_r[11:0] = r2_data[11:0];
	 	3'h3: rb_data_r[11:0] = r3_data[11:0];
	 	3'h4: rb_data_r[11:0] = r4_data[11:0];
	 	3'h5: rb_data_r[11:0] = r5_data[11:0];
	 	3'h6: rb_data_r[11:0] = r6_data[11:0];
	 	3'h7: rb_data_r[11:0] = r7_data[11:0];
	 	default: rb_data_r[11:0] = 12'b0;
	 endcase	 	
end
	 	

	            
endmodule 
